export class AbstractElement extends HTMLElement {
    constructor(renderFunc, shadow = false, mode = 'open') {
        super();
        this.connected = false;
        this.attr = {};
        const attach = (container) => {
            if (this.connected) {
                renderFunc(container, this.render());
            }
        };
        if (shadow) {
            this._attach = attach.bind(null, this.attachShadow({ mode }));
        }
        else {
            this._attach = attach.bind(null, this);
        }
    }
    set state(newState) {
        this._state = newState || this.state;
        this._attach();
    }
    get state() {
        return this._state;
    }
    static get observedAttributes() {
        return Object.keys(this.attributes).map(key => this.attributes[key]);
    }
    connectedCallback() {
        this.connected = true;
        this._attach();
    }
    attributeChangedCallback(name, oldValue, newValue) {
        if (oldValue !== newValue) {
            if (this.attr[name] !== newValue) {
                this.attr[name] = newValue;
            }
            for (const [attrProp, attrKey] of Object.entries(this.constructor['attributes'])) {
                if (attrKey === name && this[attrProp] !== newValue) {
                    this[attrProp] = newValue;
                    break;
                }
            }
            this._attach();
        }
    }
    forceUpdate() {
        this._attach();
    }
    _attach() { }
}
AbstractElement.attributes = {};
//# sourceMappingURL=abstract-element.js.map