export { AbstractElement } from './abstract-element';
export function Define(nameTag) {
    return (originalConstructor) => {
        try {
            customElements.define(nameTag, originalConstructor);
        }
        catch (error) {
            console.warn(error);
        }
    };
}
export function state() {
    return function (target, key, descriptor) {
        makePropertyMapper(target, key, (value) => { return value; }, descriptor);
    };
}
export function attr(name) {
    return function (target, key, descriptor) {
        const attributes = Reflect.get(target.constructor, 'attributes');
        Reflect.defineProperty(target.constructor, 'attributes', {
            value: Object.assign({}, attributes, { [key]: name }),
            enumerable: true,
            writable: true
        });
    };
}
function makePropertyMapper(prototype, key, mapper, descriptor) {
    if (descriptor) {
        const setter = descriptor.set || function () { };
        descriptor.set = function (val) {
            setter.apply(this, [val]);
            this.forceUpdate();
        };
    }
    else {
        const values = new Map();
        Reflect.defineProperty(prototype, key, {
            set(firstValue) {
                Reflect.defineProperty(this, key, {
                    get() {
                        return values.get(this);
                    },
                    set(value) {
                        values.set(this, mapper(value));
                        this.forceUpdate();
                    },
                    enumerable: true,
                });
                this[key] = firstValue;
            },
            enumerable: true,
            configurable: true,
        });
    }
}
//# sourceMappingURL=index.js.map